@@ -26,10 +26,18 @@ use PHPMD\Rule\TraitAware;
 use PHPMD\Utility\Strings;
 
 /**
- * This rule checks if an interface or class name exceeds the configured length excluding certain configured suffixes
+ * This rule checks if an interface or class name exceeds the configured length
+ * excluding certain configured prefixes and suffixes
  */
 class LongClassName extends AbstractRule implements ClassAware, InterfaceAware, TraitAware, EnumAware
 {
+    /**
+     * Temporary cache of configured prefixes to subtract
+     *
+     * @var string[]|null
+     */
+    protected $subtractPrefixes;
+
     /**
      * Temporary cache of configured suffixes to subtract
      *
@@ -47,12 +55,35 @@ class LongClassName extends AbstractRule implements ClassAware, InterfaceAware,
     {
         $threshold = $this->getIntProperty('maximum');
         $classOrInterfaceName = $node->getName();
-        if (Strings::lengthWithoutSuffixes($classOrInterfaceName, $this->getSubtractSuffixList()) <= $threshold) {
+        $length = Strings::lengthWithoutPrefixesAndSuffixes(
+            $classOrInterfaceName,
+            $this->getSubtractPrefixList(),
+            $this->getSubtractSuffixList()
+        );
+
+        if ($length <= $threshold) {
             return;
         }
         $this->addViolation($node, array($classOrInterfaceName, $threshold));
     }
 
+    /**
+     * Gets array of prefixes from property
+     *
+     * @return string[]
+     */
+    protected function getSubtractPrefixList()
+    {
+        if ($this->subtractPrefixes === null) {
+            $this->subtractPrefixes = Strings::splitToList(
+                $this->getStringProperty('subtract-prefixes', ''),
+                ','
+            );
+        }
+
+        return $this->subtractPrefixes;
+    }
+
     /**
      * Gets array of suffixes from property
      *
