@@ -17,6 +17,9 @@
 
 namespace PHPMD;
 
+use PHPMD\Exception\RuleClassFileNotFoundException;
+use PHPMD\Exception\RuleClassNotFoundException;
+use PHPMD\Exception\RuleSetNotFoundException;
 use RuntimeException;
 
 /**
@@ -245,16 +248,20 @@ class RuleSetFactory
      */
     private function parseRuleNode(RuleSet $ruleSet, \SimpleXMLElement $node)
     {
-        if ($node['ref'] !== null && substr($node['ref'], -3, 3) === 'xml') {
-            $this->parseRuleSetReferenceNode($ruleSet, $node);
+        $ref = (string)$node['ref'];
 
-            return;
-        }
-        if ('' === (string)$node['ref']) {
+        if ($ref === '') {
             $this->parseSingleRuleNode($ruleSet, $node);
 
             return;
         }
+
+        if (substr($ref, -3, 3) === 'xml') {
+            $this->parseRuleSetReferenceNode($ruleSet, $node);
+
+            return;
+        }
+
         $this->parseRuleReferenceNode($ruleSet, $node);
     }
 
@@ -324,7 +331,7 @@ class RuleSetFactory
      */
     private function parseSingleRuleNode(RuleSet $ruleSet, \SimpleXMLElement $ruleNode)
     {
-        $fileName = "";
+        $fileName = '';
 
         $ruleSetFolderPath = dirname($ruleSet->getFileName());
 
