@@ -19,16 +19,21 @@ namespace PHPMD\TextUI;
 
 use InvalidArgumentException;
 use PHPMD\Baseline\BaselineMode;
+use PHPMD\Cache\Model\ResultCacheStrategy;
+use PHPMD\Console\OutputInterface;
 use PHPMD\Renderer\AnsiRenderer;
+use PHPMD\Renderer\CheckStyleRenderer;
 use PHPMD\Renderer\GitHubRenderer;
 use PHPMD\Renderer\GitLabRenderer;
 use PHPMD\Renderer\HTMLRenderer;
 use PHPMD\Renderer\JSONRenderer;
+use PHPMD\Renderer\Option\Color;
+use PHPMD\Renderer\Option\Verbose;
 use PHPMD\Renderer\SARIFRenderer;
 use PHPMD\Renderer\TextRenderer;
-use PHPMD\Renderer\CheckStyleRenderer;
 use PHPMD\Renderer\XMLRenderer;
 use PHPMD\Rule;
+use PHPMD\Utility\ArgumentsValidator;
 
 /**
  * This is a helper class that collects the specified cli arguments and puts them
@@ -78,6 +83,13 @@ class CommandLineOptions
      */
     protected $reportFile;
 
+    /**
+     * An optional filename to collect errors.
+     *
+     * @var string
+     */
+    protected $errorFile;
+
     /**
      * Additional report files.
      *
@@ -85,6 +97,13 @@ class CommandLineOptions
      */
     protected $reportFiles = array();
 
+    /**
+     * List of deprecations.
+     *
+     * @var array
+     */
+    protected $deprecations = array();
+
     /**
      * A ruleset filename or a comma-separated string of ruleset filenames.
      *
@@ -130,6 +149,9 @@ class CommandLineOptions
      */
     protected $strict = false;
 
+    /** @var int */
+    protected $verbosity = OutputInterface::VERBOSITY_NORMAL;
+
     /**
      * Should PHPMD exit without error code even if error is found?
      *
@@ -165,6 +187,37 @@ class CommandLineOptions
      */
     protected $baselineFile;
 
+    /**
+     * Should PHPMD read or write the result cache state from the cache file
+     * @var bool
+     */
+    protected $cacheEnabled = false;
+
+    /**
+     * If set the path to read and write the result cache state from and to.
+     * @var string|null
+     */
+    protected $cacheFile;
+
+    /**
+     * If set determine the cache strategy. Either `content` or `timestamp`. Defaults to `content`.
+     * @var string|null
+     */
+    protected $cacheStrategy;
+
+    /**
+     * Either the output should be colored.
+     *
+     * @var bool
+     */
+    protected $colored = false;
+
+    /**
+     * Specify how many extra lines are added to a code snippet
+     * @var int|null
+     */
+    protected $extraLineInExcerpt;
+
     /**
      * Constructs a new command line options instance.
      *
@@ -177,71 +230,125 @@ class CommandLineOptions
         // Remove current file name
         array_shift($args);
 
+        $originalArguments = $args;
         $this->availableRuleSets = $availableRuleSets;
 
         $arguments = array();
+        $listenOptions = true;
+        $hasImplicitArguments = false;
+
         while (($arg = array_shift($args)) !== null) {
-            switch ($arg) {
+            if (!$listenOptions) {
+                $arguments[] = $arg;
+
+                continue;
+            }
+
+            $equalChunk = explode('=', $arg, 2);
+
+            switch ($equalChunk[0]) {
+                case '--':
+                    $this->refuseValue($equalChunk);
+                    $listenOptions = false;
+                    break;
+                case '--verbose':
+                case '-v':
+                    $this->refuseValue($equalChunk);
+                    $this->verbosity = OutputInterface::VERBOSITY_VERBOSE;
+                    break;
+                case '-vv':
+                    $this->refuseValue($equalChunk);
+                    $this->verbosity = OutputInterface::VERBOSITY_VERY_VERBOSE;
+                    break;
+                case '-vvv':
+                    $this->refuseValue($equalChunk);
+                    $this->verbosity = OutputInterface::VERBOSITY_DEBUG;
+                    break;
                 case '--min-priority':
                 case '--minimum-priority':
                 case '--minimumpriority':
-                    $this->minimumPriority = (int)array_shift($args);
+                    $this->minimumPriority = (int)$this->readValue($equalChunk, $args);
                     break;
                 case '--max-priority':
                 case '--maximum-priority':
                 case '--maximumpriority':
-                    $this->maximumPriority = (int)array_shift($args);
+                    $this->maximumPriority = (int)$this->readValue($equalChunk, $args);
                     break;
                 case '--report-file':
                 case '--reportfile':
-                    $this->reportFile = array_shift($args);
+                    $this->reportFile = $this->readValue($equalChunk, $args);
+                    break;
+                case '--error-file':
+                case '--errorfile':
+                    $this->errorFile = $this->readValue($equalChunk, $args);
                     break;
                 case '--input-file':
                 case '--inputfile':
-                    array_unshift($arguments, $this->readInputFile(array_shift($args)));
+                    array_unshift($arguments, $this->readInputFile($this->readValue($equalChunk, $args)));
                     break;
                 case '--coverage':
-                    $this->coverageReport = array_shift($args);
+                    $this->coverageReport = $this->readValue($equalChunk, $args);
                     break;
                 case '--extensions':
                     $this->logDeprecated('extensions', 'suffixes');
                     /* Deprecated: We use the suffixes option now */
-                    $this->extensions = array_shift($args);
+                    $this->extensions = $this->readValue($equalChunk, $args);
                     break;
                 case '--suffixes':
-                    $this->extensions = array_shift($args);
+                    $this->extensions = $this->readValue($equalChunk, $args);
                     break;
                 case '--ignore':
                     $this->logDeprecated('ignore', 'exclude');
                     /* Deprecated: We use the exclude option now */
-                    $this->ignore = array_shift($args);
+                    $this->ignore = $this->readValue($equalChunk, $args);
                     break;
                 case '--exclude':
-                    $this->ignore = array_shift($args);
+                    $this->ignore = $this->readValue($equalChunk, $args);
+                    break;
+                case '--color':
+                    $this->refuseValue($equalChunk);
+                    $this->colored = true;
                     break;
                 case '--version':
+                    $this->refuseValue($equalChunk);
                     $this->version = true;
 
                     return;
                 case '--strict':
+                    $this->refuseValue($equalChunk);
                     $this->strict = true;
                     break;
                 case '--not-strict':
+                    $this->refuseValue($equalChunk);
                     $this->strict = false;
                     break;
                 case '--generate-baseline':
+                    $this->refuseValue($equalChunk);
                     $this->generateBaseline = BaselineMode::GENERATE;
                     break;
                 case '--update-baseline':
+                    $this->refuseValue($equalChunk);
                     $this->generateBaseline = BaselineMode::UPDATE;
                     break;
                 case '--baseline-file':
-                    $this->baselineFile = array_shift($args);
+                    $this->baselineFile = $this->readValue($equalChunk, $args);
+                    break;
+                case '--cache':
+                    $this->refuseValue($equalChunk);
+                    $this->cacheEnabled = true;
+                    break;
+                case '--cache-file':
+                    $this->cacheFile = $this->readValue($equalChunk, $args);
+                    break;
+                case '--cache-strategy':
+                    $this->cacheStrategy = $this->readValue($equalChunk, $args);
                     break;
                 case '--ignore-errors-on-exit':
+                    $this->refuseValue($equalChunk);
                     $this->ignoreErrorsOnExit = true;
                     break;
                 case '--ignore-violations-on-exit':
+                    $this->refuseValue($equalChunk);
                     $this->ignoreViolationsOnExit = true;
                     break;
                 case '--reportfile-checkstyle':
@@ -253,9 +360,13 @@ class CommandLineOptions
                 case '--reportfile-text':
                 case '--reportfile-xml':
                     preg_match('(^\-\-reportfile\-(checkstyle|github|gitlab|html|json|sarif|text|xml)$)', $arg, $match);
-                    $this->reportFiles[$match[1]] = array_shift($args);
+                    $this->reportFiles[$match[1]] = $this->readValue($equalChunk, $args);
+                    break;
+                case '--extra-line-in-excerpt':
+                    $this->extraLineInExcerpt = (int)$this->readValue($equalChunk, $args);
                     break;
                 default:
+                    $hasImplicitArguments = true;
                     $arguments[] = $arg;
                     break;
             }
@@ -265,9 +376,25 @@ class CommandLineOptions
             throw new InvalidArgumentException($this->usage(), self::INPUT_ERROR);
         }
 
-        $this->inputPath    = (string)array_shift($arguments);
-        $this->reportFormat = (string)array_shift($arguments);
-        $this->ruleSets     = (string)array_shift($arguments);
+        $validator = new ArgumentsValidator($hasImplicitArguments, $originalArguments, $arguments);
+
+        $this->ruleSets = (string)array_pop($arguments);
+        $validator->validate('ruleset', $this->ruleSets);
+
+        $this->reportFormat = (string)array_pop($arguments);
+        $validator->validate('report format', $this->reportFormat);
+
+        $this->inputPath = implode(',', $arguments);
+
+        if ($this->inputPath === '-') {
+            $this->inputPath = 'php://stdin';
+
+            return;
+        }
+
+        foreach ($arguments as $arg) {
+            $validator->validate('input path', $arg);
+        }
     }
 
     /**
@@ -301,6 +428,27 @@ class CommandLineOptions
         return $this->reportFile;
     }
 
+    /**
+     * Returns the output filename for the errors or <b>null</b> when
+     * the report should be displayed in STDERR.
+     *
+     * @return string
+     */
+    public function getErrorFile()
+    {
+        return $this->errorFile;
+    }
+
+    /**
+     * Return the list of deprecations raised when parsing options.
+     *
+     * @return list<string>
+     */
+    public function getDeprecations()
+    {
+        return $this->deprecations;
+    }
+
     /**
      * Returns a hash with report files specified for different renderers. The
      * key represents the report format and the value the report file location.
@@ -396,6 +544,14 @@ class CommandLineOptions
         return $this->strict;
     }
 
+    /**
+     * @return int
+     */
+    public function getVerbosity()
+    {
+        return $this->verbosity;
+    }
+
     /**
      * Should the current violations be baselined
      *
@@ -416,6 +572,41 @@ class CommandLineOptions
         return $this->baselineFile;
     }
 
+    /**
+     * @return bool
+     */
+    public function isCacheEnabled()
+    {
+        return $this->cacheEnabled;
+    }
+
+    /**
+     * The filepath to the result cache state file
+     *
+     * @return string
+     */
+    public function cacheFile()
+    {
+        return $this->cacheFile === null ? '.phpmd.result-cache.php' : $this->cacheFile;
+    }
+
+    /**
+     * The caching strategy to determine if a file should be (re)inspected. Either
+     * `content` or last modified `timestamp` based.
+     *
+     * @return string
+     */
+    public function cacheStrategy()
+    {
+        switch ($this->cacheStrategy) {
+            case ResultCacheStrategy::CONTENT:
+            case ResultCacheStrategy::TIMESTAMP:
+                return $this->cacheStrategy;
+            default:
+                return ResultCacheStrategy::CONTENT;
+        }
+    }
+
     /**
      * Was the <b>--ignore-errors-on-exit</b> passed to PHPMD's command line interface?
      *
@@ -437,6 +628,15 @@ class CommandLineOptions
         return $this->ignoreViolationsOnExit;
     }
 
+    /**
+     * Specify how many extra lines are added to a code snippet
+     *
+     * @return int|null
+     */
+    public function extraLineInExcerpt()
+    {
+        return $this->extraLineInExcerpt;
+    }
     /**
      * Creates a report renderer instance based on the user's command line
      * argument.
@@ -454,6 +654,26 @@ class CommandLineOptions
      * @throws InvalidArgumentException When the specified renderer does not exist.
      */
     public function createRenderer($reportFormat = null)
+    {
+        $renderer = $this->createRendererWithoutOptions($reportFormat);
+
+        if ($renderer instanceof Verbose) {
+            $renderer->setVerbosityLevel($this->verbosity);
+        }
+
+        if ($renderer instanceof Color) {
+            $renderer->setColored($this->colored);
+        }
+
+        return $renderer;
+    }
+
+    /**
+     * @param string $reportFormat
+     * @return \PHPMD\AbstractRenderer
+     * @throws InvalidArgumentException When the specified renderer does not exist.
+     */
+    protected function createRendererWithoutOptions($reportFormat = null)
     {
         $reportFormat = $reportFormat ?: $this->reportFormat;
 
@@ -526,7 +746,7 @@ class CommandLineOptions
      */
     protected function createHtmlRenderer()
     {
-        return new HTMLRenderer();
+        return new HTMLRenderer($this->extraLineInExcerpt);
     }
 
     /**
@@ -538,7 +758,7 @@ class CommandLineOptions
     }
 
     /**
-     * @return \PHPMD\Renderer\JSONRenderer
+     * @return \PHPMD\Renderer\CheckStyleRenderer
      */
     protected function createCheckStyleRenderer()
     {
@@ -598,21 +818,26 @@ class CommandLineOptions
     public function usage()
     {
         $availableRenderers = $this->getListOfAvailableRenderers();
+        $noRenderers = ($availableRenderers === null);
 
         return 'Mandatory arguments:' . \PHP_EOL .
             '1) A php source code filename or directory. Can be a comma-' .
-            'separated string' . \PHP_EOL .
+            'separated string, glob pattern, or "-" to scan stdin' . \PHP_EOL .
             '2) A report format' . \PHP_EOL .
             '3) A ruleset filename or a comma-separated string of ruleset' .
             'filenames' . \PHP_EOL . \PHP_EOL .
             'Example: phpmd /path/to/source format ruleset' . \PHP_EOL . \PHP_EOL .
-            'Available formats: ' . $availableRenderers . '.' . \PHP_EOL .
+            ($noRenderers ? 'No available formats' : 'Available formats: ' . $availableRenderers) . '.' . \PHP_EOL .
             'Available rulesets: ' . implode(', ', $this->availableRuleSets) . '.' . \PHP_EOL . \PHP_EOL .
             'Optional arguments that may be put after the mandatory arguments:' .
             \PHP_EOL .
-            '--minimumpriority: rule priority threshold; rules with lower ' .
+            '--verbose, -v, -vv, -vvv: Show debug information.' . \PHP_EOL .
+            '--minimum-priority: rule priority threshold; rules with lower ' .
             'priority than this will not be used' . \PHP_EOL .
-            '--reportfile: send report output to a file; default to STDOUT' .
+            '--report-file: send report output to a file; default to STDOUT' .
+            \PHP_EOL .
+            '--error-file: send errors (other than reported violations) ' .
+            'output to a file; default to STDERR' .
             \PHP_EOL .
             '--suffixes: comma-separated string of valid source code ' .
             'filename extensions, e.g. php,phtml' . \PHP_EOL .
@@ -625,16 +850,28 @@ class CommandLineOptions
             'even on error' . \PHP_EOL .
             '--ignore-violations-on-exit: will exit with a zero code, ' .
             'even if any violations are found' . \PHP_EOL .
+            '--cache: will enable the result cache.' . \PHP_EOL .
+            '--cache-file: instead of the default .phpmd.result-cache.php' .
+            ' will use this file as result cache file path.' . \PHP_EOL .
+            '--cache-strategy: sets the caching strategy to determine if' .
+            ' a file is still fresh. Either `content` to base it on the ' .
+            'file contents, or `timestamp` to base it on the file modified ' .
+            'timestamp' . \PHP_EOL .
             '--generate-baseline: will generate a phpmd.baseline.xml next ' .
             'to the first ruleset file location' . \PHP_EOL .
             '--update-baseline: will remove any non-existing violations from the phpmd.baseline.xml' . \PHP_EOL .
-            '--baseline-file: a custom location of the baseline file' . \PHP_EOL;
+            '--baseline-file: a custom location of the baseline file' . \PHP_EOL .
+            '--color: enable color in output' . \PHP_EOL .
+            '--extra-line-in-excerpt: Specify how many extra lines are added ' .
+            'to a code snippet in html format' . \PHP_EOL .
+            '--: Explicit argument separator: Anything after "--" will be read as an argument even if' .
+            'it starts with "-" or matches the name of an option' . \PHP_EOL;
     }
 
     /**
      * Get a list of available renderers
      *
-     * @return string The list of renderers found.
+     * @return string|null The list of renderers found separated by comma, or null if none.
      */
     protected function getListOfAvailableRenderers()
     {
@@ -650,11 +887,7 @@ class CommandLineOptions
 
         sort($renderers);
 
-        if (count($renderers) > 1) {
-            return implode(', ', $renderers);
-        }
-
-        return array_pop($renderers);
+        return implode(', ', $renderers) ?: null;
     }
 
     /**
@@ -666,13 +899,11 @@ class CommandLineOptions
      */
     protected function logDeprecated($deprecatedName, $newName)
     {
-        $message = sprintf(
+        $this->deprecations[] = sprintf(
             'The --%s option is deprecated, please use --%s instead.',
             $deprecatedName,
             $newName
         );
-
-        fwrite(STDERR, $message . PHP_EOL . PHP_EOL);
     }
 
     /**
@@ -693,4 +924,35 @@ class CommandLineOptions
         }
         throw new InvalidArgumentException("Input file '{$inputFile}' not exists.");
     }
+
+    /**
+     * Throw an exception if a boolean option has a value (is followed by equal).
+     *
+     * @param string[] $equalChunk The CLI parameter split in 2 by "=" sign
+     *
+     * @throws InvalidArgumentException if a boolean option has a value (is followed by equal)
+     */
+    private function refuseValue(array $equalChunk)
+    {
+        if (count($equalChunk) > 1) {
+            throw new InvalidArgumentException($equalChunk[0] . ' option does not accept a value');
+        }
+    }
+
+    /**
+     * Return value for an option either what is after "=" sign if present, else take the next CLI parameter.
+     *
+     * @param string[] $equalChunk The CLI parameter split in 2 by "=" sign
+     * @param string[] &$args      The remaining CLI parameters not yet parsed
+     *
+     * @return string|null
+     */
+    private function readValue(array $equalChunk, array &$args)
+    {
+        if (count($equalChunk) > 1) {
+            return $equalChunk[1];
+        }
+
+        return array_shift($args);
+    }
 }
