@@ -27,6 +27,18 @@ class Paths
         return $filePath;
     }
 
+    /**
+     * Concat pathB to pathA with a single path separators between them
+     *
+     * @param string $pathA
+     * @param string $pathB
+     * @return string
+     */
+    public static function concat($pathA, $pathB)
+    {
+        return rtrim(str_replace('\\', '/', $pathA), '/') . '/' . ltrim(str_replace('\\', '/', $pathB), '/');
+    }
+
     /**
      * Get the realpath of the given path or exception on failure
      * @param string $path
