@@ -34,6 +34,17 @@ class PathsTest extends AbstractTest
         static::assertSame('/foo/bar/text.txt', Paths::getRelativePath('/bar', '/foo/bar/text.txt'));
     }
 
+    /**
+     * @covers ::concat
+     */
+    public function testConcat()
+    {
+        static::assertSame('pathA/pathB', Paths::concat('pathA', 'pathB'));
+        static::assertSame('pathA/pathB', Paths::concat('pathA', '/pathB'));
+        static::assertSame('pathA/pathB', Paths::concat('pathA/', '/pathB'));
+        static::assertSame('/file/pathA/pathB/example.txt', Paths::concat('\\file\\pathA\\', '\\pathB\\example.txt'));
+    }
+
     /**
      * @covers ::getRealPath
      */
